#!/usr/bin/env ruby

=begin
=Command gdir_server.rb 

A stand-alone server program to provide a directory service of GPHys
objects by using GDir.

==Usage

At the  command line,

   % gdir_service.rb top_dir [port] [-a allowed_address]..

where 
* top_dir: can be ".", "/", or whatever directory in your files system.
* port: the port number you want to assign (internally determined
  if not specified). A complete URI is printed if the server is started.
* -a: limits accesses from remote hosts only to the ones specified.
  EXAMPLE:
    -a '192.168.1.*'
    -a hoge.hero.org
=end

require "drb/drb"
require "drb/timeridconv"
require "drb/acl"
require "numru/gdir"

DRb.install_id_conv(DRb::TimerIdConv.new(1200))

module NumRu
  class GDir
    include DRbUndumped
  end
end

class NArray
  def self._load(o) to_na(*Marshal::load(o)).ntoh end
  def _dump(limit) 
    Marshal::dump([hton.to_s, typecode, *shape]) 
  end
end

include NumRu

usage = <<EOS


USAGE: % #{$0} top_dir [port] [-a allowed_address]..

where
* top_dir: can be ".", "/", or whatever directory in your files system.
* port: the port number you want to assign (internally determined
  if not specified). A complete URI is printed if the server is started.
* -a: limits accesses from remote hosts only to the ones specified.
  Can be repeated multiple times.
  EXAMPLE:
    -a '192.168.1.*' -a hoge.hero.org
EOS

# <interpret comman-line arguments>

if ARGV.include?('-a')
  # Access control
  list = ['deny', 'all']
  while(idx=ARGV.index('-a'))
    ARGV.delete_at(idx) 
    begin
      list.push( 'allow', ARGV.delete_at(idx) )
    rescue
      raise usage
    end
  end
  acl = ACL.new(list)
  DRb.install_acl(acl)
end

top = ARGV.shift || raise(usage)
port = ARGV.shift
##top = top.dup.untaint    # unfrozen,untaint
##port = port.dup.untaint  # unfrozen,untaint
##$SAFE=1
print usage if port && port.to_i.to_s != port    # make sure it is an integer

# <set the top & direcotry>
begin
  GDir.top = top
  rootdir = GDir.new('/')
rescue
  print "\n",usage,"\n"
  raise
end

# <start the server>
uri_seed = ( port ? 'druby://:'+port : nil )
DRb.start_service(uri_seed, rootdir)
puts 'URI: '+DRb.uri
puts '[return] to exit'
gets
